from distutils.core import setup
setup(
  name = 'pzgram',
  packages = ['pzgram'], # this must be the same as the name above
  version = '0.2',
  description = 'A pz-way to create your telegram bot',
  author = 'Giovani Casari',
  author_email = 'casari.giovanni@gmail.com',
  url = 'https://github.com/infopz/pzgram', # use the URL to the github repo
  download_url = 'https://github.com/infopz/pzgram/archive/0.2.tar.gz', # I'll explain this in a second
  keywords = ['telegram', 'telegram-bot', 'telegram-api'], # arbitrary keywords
  classifiers = [],
)